@@ -377,6 +377,125 @@ def test_add_static(router):
     assert len(resource) == 2
 
 
+def test_add_static_append_version(router):
+    resource = router.add_static('/st',
+                                 os.path.dirname(__file__),
+                                 name='static')
+    url = resource.url(filename='/data.unknown_mime_type', append_version=True)
+    expect_url = '/st/data.unknown_mime_type?' \
+                 'v=aUsn8CHEhhszc81d28QmlcBW0KQpfS2F4trgQKhOYd8%3D'
+    assert expect_url == url
+
+
+def test_add_static_append_version_set_from_constructor(router):
+    resource = router.add_static('/st',
+                                 os.path.dirname(__file__),
+                                 append_version=True,
+                                 name='static')
+    url = resource.url(filename='/data.unknown_mime_type')
+    expect_url = '/st/data.unknown_mime_type?' \
+                 'v=aUsn8CHEhhszc81d28QmlcBW0KQpfS2F4trgQKhOYd8%3D'
+    assert expect_url == url
+
+
+def test_add_static_append_version_override_constructor(router):
+    resource = router.add_static('/st',
+                                 os.path.dirname(__file__),
+                                 append_version=True,
+                                 name='static')
+    url = resource.url(filename='/data.unknown_mime_type',
+                       append_version=False)
+    expect_url = '/st/data.unknown_mime_type'
+    assert expect_url == url
+
+
+def test_add_static_append_version_filename_without_slash(router):
+    resource = router.add_static('/st',
+                                 os.path.dirname(__file__),
+                                 name='static')
+    url = resource.url(filename='data.unknown_mime_type', append_version=True)
+    expect_url = '/st/data.unknown_mime_type?' \
+                 'v=aUsn8CHEhhszc81d28QmlcBW0KQpfS2F4trgQKhOYd8%3D'
+    assert expect_url == url
+
+
+def test_add_static_append_version_non_exists_file(router):
+    resource = router.add_static('/st',
+                                 os.path.dirname(__file__),
+                                 name='static')
+    url = resource.url(filename='/non_exists_file', append_version=True)
+    assert '/st/non_exists_file' == url
+
+
+def test_add_static_append_version_non_exists_file_without_slash(router):
+    resource = router.add_static('/st',
+                                 os.path.dirname(__file__),
+                                 name='static')
+    url = resource.url(filename='non_exists_file', append_version=True)
+    assert '/st/non_exists_file' == url
+
+
+def test_add_static_append_version_follow_symlink(router, tmpdir):
+    """
+    Tests the access to a symlink, in static folder with apeend_version
+    """
+    tmp_dir_path = str(tmpdir)
+    symlink_path = os.path.join(tmp_dir_path, 'append_version_symlink')
+    symlink_target_path = os.path.dirname(__file__)
+    os.symlink(symlink_target_path, symlink_path, True)
+
+    # Register global static route:
+    resource = router.add_static('/st', tmp_dir_path, follow_symlinks=True,
+                                 append_version=True)
+
+    url = resource.url(
+        filename='/append_version_symlink/data.unknown_mime_type')
+
+    expect_url = '/st/append_version_symlink/data.unknown_mime_type?' \
+                 'v=aUsn8CHEhhszc81d28QmlcBW0KQpfS2F4trgQKhOYd8%3D'
+    assert expect_url == url
+
+
+def test_add_static_append_version_not_follow_symlink(router, tmpdir):
+    """
+    Tests the access to a symlink, in static folder with apeend_version
+    """
+    tmp_dir_path = str(tmpdir)
+    symlink_path = os.path.join(tmp_dir_path, 'append_version_symlink')
+    symlink_target_path = os.path.dirname(__file__)
+    os.symlink(symlink_target_path, symlink_path, True)
+
+    # Register global static route:
+    resource = router.add_static('/st', tmp_dir_path, follow_symlinks=False,
+                                 append_version=True)
+
+    filename = '/append_version_symlink/data.unknown_mime_type'
+    url = resource.url(filename=filename)
+    assert '/st/append_version_symlink/data.unknown_mime_type' == url
+
+
+def test_add_static_append_version_with_query(router):
+    resource = router.add_static('/st',
+                                 os.path.dirname(__file__),
+                                 name='static')
+    url = resource.url(filename='/data.unknown_mime_type',
+                       append_version=True,
+                       query={'key': 'val'})
+    expect_url = '/st/data.unknown_mime_type?' \
+                 'v=aUsn8CHEhhszc81d28QmlcBW0KQpfS2F4trgQKhOYd8%3D&key=val'
+    assert expect_url == url
+
+
+def test_add_static_append_version_non_exists_file_with_query(router):
+    resource = router.add_static('/st',
+                                 os.path.dirname(__file__),
+                                 name='static')
+    url = resource.url(filename='/non_exists_file',
+                       append_version=True,
+                       query={'key': 'val'})
+    assert '/st/non_exists_file?key=val' == url
+
+
 def test_plain_not_match(router):
     handler = make_handler()
     router.add_route('GET', '/get/path', handler, name='name')
@@ -1049,3 +1168,11 @@ def test_convert_empty_path_to_slash_on_freezing(router):
     assert resource.get_info() == {'path': ''}
     router.freeze()
     assert resource.get_info() == {'path': '/'}
+
+
+def test_deprecate_non_coroutine(router):
+    def handler(request):
+        pass
+
+    with pytest.warns(DeprecationWarning):
+        router.add_route('GET', '/handler', handler)
