@@ -1,4 +1,5 @@
 import asyncio
+import socket
 from collections import MutableMapping
 from unittest import mock
 
@@ -12,17 +13,12 @@ from aiohttp.test_utils import make_mocked_request
 from aiohttp.web import HTTPRequestEntityTooLarge
 
 
-@pytest.fixture
-def make_request():
-    return make_mocked_request
-
-
-def test_ctor(make_request):
-    req = make_request('GET', '/path/to?a=1&b=2')
+def test_ctor():
+    req = make_mocked_request('GET', '/path/to?a=1&b=2')
 
     assert 'GET' == req.method
     assert HttpVersion(1, 1) == req.version
-    assert req.host is None
+    assert req.host == socket.getfqdn()
     assert '/path/to?a=1&b=2' == req.path_qs
     assert '/path/to' == req.path
     assert 'a=1&b=2' == req.query_string
@@ -42,97 +38,98 @@ def test_ctor(make_request):
     payload = mock.Mock()
     protocol = mock.Mock()
     app = mock.Mock()
-    req = make_request('GET', '/path/to?a=1&b=2', headers=headers,
-                       protocol=protocol, payload=payload, app=app)
+    req = make_mocked_request('GET', '/path/to?a=1&b=2', headers=headers,
+                              protocol=protocol, payload=payload, app=app)
     assert req.app is app
     assert req.content is payload
     assert req.protocol is protocol
     assert req.transport is protocol.transport
     assert req.headers == headers
-    assert req.raw_headers == ((b'Foo', b'bar'),)
+    assert req.raw_headers == ((b'FOO', b'bar'),)
     assert req.task is req._task
 
     with pytest.warns(DeprecationWarning):
         assert req.GET is req.query
 
 
-def test_doubleslashes(make_request):
+def test_doubleslashes():
     # NB: //foo/bar is an absolute URL with foo netloc and /bar path
-    req = make_request('GET', '/bar//foo/')
+    req = make_mocked_request('GET', '/bar//foo/')
     assert '/bar//foo/' == req.path
 
 
-def test_content_type_not_specified(make_request):
-    req = make_request('Get', '/')
+def test_content_type_not_specified():
+    req = make_mocked_request('Get', '/')
     assert 'application/octet-stream' == req.content_type
 
 
-def test_content_type_from_spec(make_request):
-    req = make_request('Get', '/',
-                       CIMultiDict([('CONTENT-TYPE', 'application/json')]))
+def test_content_type_from_spec():
+    req = make_mocked_request('Get', '/',
+                              CIMultiDict([('CONTENT-TYPE',
+                                            'application/json')]))
     assert 'application/json' == req.content_type
 
 
-def test_content_type_from_spec_with_charset(make_request):
-    req = make_request(
+def test_content_type_from_spec_with_charset():
+    req = make_mocked_request(
         'Get', '/',
         CIMultiDict([('CONTENT-TYPE', 'text/html; charset=UTF-8')]))
     assert 'text/html' == req.content_type
     assert 'UTF-8' == req.charset
 
 
-def test_calc_content_type_on_getting_charset(make_request):
-    req = make_request(
+def test_calc_content_type_on_getting_charset():
+    req = make_mocked_request(
         'Get', '/',
         CIMultiDict([('CONTENT-TYPE', 'text/html; charset=UTF-8')]))
     assert 'UTF-8' == req.charset
     assert 'text/html' == req.content_type
 
 
-def test_urlencoded_querystring(make_request):
-    req = make_request('GET',
-                       '/yandsearch?text=%D1%82%D0%B5%D0%BA%D1%81%D1%82')
+def test_urlencoded_querystring():
+    req = make_mocked_request(
+        'GET', '/yandsearch?text=%D1%82%D0%B5%D0%BA%D1%81%D1%82')
     assert {'text': 'текст'} == req.query
 
 
-def test_non_ascii_path(make_request):
-    req = make_request('GET', '/путь')
+def test_non_ascii_path():
+    req = make_mocked_request('GET', '/путь')
     assert '/путь' == req.path
 
 
-def test_non_ascii_raw_path(make_request):
-    req = make_request('GET', '/путь')
+def test_non_ascii_raw_path():
+    req = make_mocked_request('GET', '/путь')
     assert '/путь' == req.raw_path
 
 
-def test_content_length(make_request):
-    req = make_request('Get', '/',
-                       CIMultiDict([('CONTENT-LENGTH', '123')]))
+def test_content_length():
+    req = make_mocked_request('Get', '/',
+                              CIMultiDict([('CONTENT-LENGTH', '123')]))
 
     assert 123 == req.content_length
 
 
-def test_non_keepalive_on_http10(make_request):
-    req = make_request('GET', '/', version=HttpVersion(1, 0))
+def test_non_keepalive_on_http10():
+    req = make_mocked_request('GET', '/', version=HttpVersion(1, 0))
     assert not req.keep_alive
 
 
-def test_non_keepalive_on_closing(make_request):
-    req = make_request('GET', '/', closing=True)
+def test_non_keepalive_on_closing():
+    req = make_mocked_request('GET', '/', closing=True)
     assert not req.keep_alive
 
 
 @asyncio.coroutine
-def test_call_POST_on_GET_request(make_request):
-    req = make_request('GET', '/')
+def test_call_POST_on_GET_request():
+    req = make_mocked_request('GET', '/')
 
     ret = yield from req.post()
     assert CIMultiDict() == ret
 
 
 @asyncio.coroutine
-def test_call_POST_on_weird_content_type(make_request):
-    req = make_request(
+def test_call_POST_on_weird_content_type():
+    req = make_mocked_request(
         'POST', '/',
         headers=CIMultiDict({'CONTENT-TYPE': 'something/weird'}))
 
@@ -141,16 +138,16 @@ def test_call_POST_on_weird_content_type(make_request):
 
 
 @asyncio.coroutine
-def test_call_POST_twice(make_request):
-    req = make_request('GET', '/')
+def test_call_POST_twice():
+    req = make_mocked_request('GET', '/')
 
     ret1 = yield from req.post()
     ret2 = yield from req.post()
     assert ret1 is ret2
 
 
-def test_no_request_cookies(make_request):
-    req = make_request('GET', '/')
+def test_no_request_cookies():
+    req = make_mocked_request('GET', '/')
 
     assert req.cookies == {}
 
@@ -158,17 +155,17 @@ def test_no_request_cookies(make_request):
     assert cookies is req.cookies
 
 
-def test_request_cookie(make_request):
+def test_request_cookie():
     headers = CIMultiDict(COOKIE='cookie1=value1; cookie2=value2')
-    req = make_request('GET', '/', headers=headers)
+    req = make_mocked_request('GET', '/', headers=headers)
 
     assert req.cookies == {'cookie1': 'value1',
                            'cookie2': 'value2'}
 
 
-def test_request_cookie__set_item(make_request):
+def test_request_cookie__set_item():
     headers = CIMultiDict(COOKIE='name=value')
-    req = make_request('GET', '/', headers=headers)
+    req = make_mocked_request('GET', '/', headers=headers)
 
     assert req.cookies == {'name': 'value'}
 
@@ -176,105 +173,94 @@ def test_request_cookie__set_item(make_request):
         req.cookies['my'] = 'value'
 
 
-def test_match_info(make_request):
-    req = make_request('GET', '/')
+def test_match_info():
+    req = make_mocked_request('GET', '/')
     assert req._match_info is req.match_info
 
 
-def test_request_is_mutable_mapping(make_request):
-    req = make_request('GET', '/')
+def test_request_is_mutable_mapping():
+    req = make_mocked_request('GET', '/')
     assert isinstance(req, MutableMapping)
     req['key'] = 'value'
     assert 'value' == req['key']
 
 
-def test_request_delitem(make_request):
-    req = make_request('GET', '/')
+def test_request_delitem():
+    req = make_mocked_request('GET', '/')
     req['key'] = 'value'
     assert 'value' == req['key']
     del req['key']
     assert 'key' not in req
 
 
-def test_request_len(make_request):
-    req = make_request('GET', '/')
+def test_request_len():
+    req = make_mocked_request('GET', '/')
     assert len(req) == 0
     req['key'] = 'value'
     assert len(req) == 1
 
 
-def test_request_iter(make_request):
-    req = make_request('GET', '/')
+def test_request_iter():
+    req = make_mocked_request('GET', '/')
     req['key'] = 'value'
     req['key2'] = 'value2'
     assert set(req) == {'key', 'key2'}
 
 
-def test___repr__(make_request):
-    req = make_request('GET', '/path/to')
+def test___repr__():
+    req = make_mocked_request('GET', '/path/to')
     assert "<Request GET /path/to >" == repr(req)
 
 
-def test___repr___non_ascii_path(make_request):
-    req = make_request('GET', '/path/\U0001f415\U0001f308')
+def test___repr___non_ascii_path():
+    req = make_mocked_request('GET', '/path/\U0001f415\U0001f308')
     assert "<Request GET /path/\\U0001f415\\U0001f308 >" == repr(req)
 
 
-def test_http_scheme(make_request):
-    req = make_request('GET', '/')
+def test_http_scheme():
+    req = make_mocked_request('GET', '/', headers={'Host': 'example.com'})
     assert "http" == req.scheme
     assert req.secure is False
 
 
-def test_https_scheme_by_ssl_transport(make_request):
-    req = make_request('GET', '/', sslcontext=True)
+def test_https_scheme_by_ssl_transport():
+    req = make_mocked_request('GET', '/', headers={'Host': 'example.com'},
+                              sslcontext=True)
     assert "https" == req.scheme
     assert req.secure is True
 
 
-def test_https_scheme_by_secure_proxy_ssl_header(make_request):
-    req = make_request('GET', '/',
-                       secure_proxy_ssl_header=('X-HEADER', '1'),
-                       headers=CIMultiDict({'X-HEADER': '1'}))
-    assert "https" == req.scheme
-    assert req.secure is True
-
-
-def test_https_scheme_by_secure_proxy_ssl_header_false_test(make_request):
-    req = make_request('GET', '/',
-                       secure_proxy_ssl_header=('X-HEADER', '1'),
-                       headers=CIMultiDict({'X-HEADER': '0'}))
-    assert "http" == req.scheme
-    assert req.secure is False
-
-
-def test_single_forwarded_header(make_request):
+def test_single_forwarded_header():
     header = 'by=identifier;for=identifier;host=identifier;proto=identifier'
-    req = make_request('GET', '/', headers=CIMultiDict({'Forwarded': header}))
+    req = make_mocked_request('GET', '/',
+                              headers=CIMultiDict({'Forwarded': header}))
     assert req.forwarded[0]['by'] == 'identifier'
     assert req.forwarded[0]['for'] == 'identifier'
     assert req.forwarded[0]['host'] == 'identifier'
     assert req.forwarded[0]['proto'] == 'identifier'
 
 
-def test_single_forwarded_header_camelcase(make_request):
+def test_single_forwarded_header_camelcase():
     header = 'bY=identifier;fOr=identifier;HOst=identifier;pRoTO=identifier'
-    req = make_request('GET', '/', headers=CIMultiDict({'Forwarded': header}))
+    req = make_mocked_request('GET', '/',
+                              headers=CIMultiDict({'Forwarded': header}))
     assert req.forwarded[0]['by'] == 'identifier'
     assert req.forwarded[0]['for'] == 'identifier'
     assert req.forwarded[0]['host'] == 'identifier'
     assert req.forwarded[0]['proto'] == 'identifier'
 
 
-def test_single_forwarded_header_single_param(make_request):
+def test_single_forwarded_header_single_param():
     header = 'BY=identifier'
-    req = make_request('GET', '/', headers=CIMultiDict({'Forwarded': header}))
+    req = make_mocked_request('GET', '/',
+                              headers=CIMultiDict({'Forwarded': header}))
     assert req.forwarded[0]['by'] == 'identifier'
 
 
-def test_single_forwarded_header_multiple_param(make_request):
+def test_single_forwarded_header_multiple_param():
     header = 'By=identifier1,BY=identifier2,  By=identifier3 ,  BY=identifier4'
-    req = make_request('GET', '/', headers=CIMultiDict({'Forwarded': header}))
+    req = make_mocked_request('GET', '/',
+                              headers=CIMultiDict({'Forwarded': header}))
     assert len(req.forwarded) == 4
     assert req.forwarded[0]['by'] == 'identifier1'
     assert req.forwarded[1]['by'] == 'identifier2'
@@ -282,18 +268,74 @@ def test_single_forwarded_header_multiple_param(make_request):
     assert req.forwarded[3]['by'] == 'identifier4'
 
 
-def test_single_forwarded_header_quoted_escaped(make_request):
+def test_single_forwarded_header_quoted_escaped():
     header = 'BY=identifier;pROTO="\lala lan\d\~ 123\!&"'
-    req = make_request('GET', '/', headers=CIMultiDict({'Forwarded': header}))
+    req = make_mocked_request('GET', '/',
+                              headers=CIMultiDict({'Forwarded': header}))
     assert req.forwarded[0]['by'] == 'identifier'
     assert req.forwarded[0]['proto'] == 'lala land~ 123!&'
 
 
-def test_multiple_forwarded_headers(make_request):
+def test_single_forwarded_header_custom_param():
+    header = r'BY=identifier;PROTO=https;SOME="other, \"value\""'
+    req = make_mocked_request('GET', '/',
+                              headers=CIMultiDict({'Forwarded': header}))
+    assert len(req.forwarded) == 1
+    assert req.forwarded[0]['by'] == 'identifier'
+    assert req.forwarded[0]['proto'] == 'https'
+    assert req.forwarded[0]['some'] == 'other, "value"'
+
+
+def test_single_forwarded_header_empty_params():
+    # This is allowed by the grammar given in RFC 7239
+    header = ';For=identifier;;PROTO=https;;;'
+    req = make_mocked_request('GET', '/',
+                              headers=CIMultiDict({'Forwarded': header}))
+    assert req.forwarded[0]['for'] == 'identifier'
+    assert req.forwarded[0]['proto'] == 'https'
+
+
+def test_single_forwarded_header_bad_separator():
+    header = 'BY=identifier PROTO=https'
+    req = make_mocked_request('GET', '/',
+                              headers=CIMultiDict({'Forwarded': header}))
+    assert 'proto' not in req.forwarded[0]
+
+
+def test_single_forwarded_header_injection1():
+    # We might receive a header like this if we're sitting behind a reverse
+    # proxy that blindly appends a forwarded-element without checking
+    # the syntax of existing field-values. We should be able to recover
+    # the appended element anyway.
+    header = 'for=_injected;by=", for=_real'
+    req = make_mocked_request('GET', '/',
+                              headers=CIMultiDict({'Forwarded': header}))
+    assert len(req.forwarded) == 2
+    assert 'by' not in req.forwarded[0]
+    assert req.forwarded[1]['for'] == '_real'
+
+
+def test_single_forwarded_header_injection2():
+    header = 'very bad syntax, for=_real'
+    req = make_mocked_request('GET', '/',
+                              headers=CIMultiDict({'Forwarded': header}))
+    assert len(req.forwarded) == 2
+    assert 'for' not in req.forwarded[0]
+    assert req.forwarded[1]['for'] == '_real'
+
+
+def test_single_forwarded_header_long_quoted_string():
+    header = 'for="' + '\\\\' * 5000 + '"'
+    req = make_mocked_request('GET', '/',
+                              headers=CIMultiDict({'Forwarded': header}))
+    assert req.forwarded[0]['for'] == '\\' * 5000
+
+
+def test_multiple_forwarded_headers():
     headers = CIMultiDict()
     headers.add('Forwarded', 'By=identifier1;for=identifier2, BY=identifier3')
     headers.add('Forwarded', 'By=identifier4;fOr=identifier5')
-    req = make_request('GET', '/', headers=headers)
+    req = make_mocked_request('GET', '/', headers=headers)
     assert len(req.forwarded) == 3
     assert req.forwarded[0]['by'] == 'identifier1'
     assert req.forwarded[0]['for'] == 'identifier2'
@@ -302,75 +344,52 @@ def test_multiple_forwarded_headers(make_request):
     assert req.forwarded[2]['for'] == 'identifier5'
 
 
-def test_https_scheme_by_forwarded_header(make_request):
-    req = make_request('GET', '/',
-                       headers=CIMultiDict(
-                           {'Forwarded': 'by=;for=;host=;proto=https'}))
-    assert "https" == req.scheme
-    assert req.secure is True
-
-
-def test_https_scheme_by_malformed_forwarded_header(make_request):
-    req = make_request('GET', '/',
-                       headers=CIMultiDict({'Forwarded': 'malformed value'}))
-    assert "http" == req.scheme
-    assert req.secure is False
-
-
-def test_https_scheme_by_x_forwarded_proto_header(make_request):
-    req = make_request('GET', '/',
-                       headers=CIMultiDict({'X-Forwarded-Proto': 'https'}))
-    assert "https" == req.scheme
-    assert req.secure is True
-
-
-def test_https_scheme_by_x_forwarded_proto_header_no_tls(make_request):
-    req = make_request('GET', '/',
-                       headers=CIMultiDict({'X-Forwarded-Proto': 'http'}))
-    assert "http" == req.scheme
-    assert req.secure is False
-
-
-def test_host_by_forwarded_header(make_request):
+def test_multiple_forwarded_headers_bad_syntax():
     headers = CIMultiDict()
-    headers.add('Forwarded', 'By=identifier1;for=identifier2, BY=identifier3')
-    headers.add('Forwarded', 'by=;for=;host=example.com')
-    req = make_request('GET', '/', headers=headers)
+    headers.add('Forwarded', 'for=_1;by=_2')
+    headers.add('Forwarded', 'invalid value')
+    headers.add('Forwarded', '')
+    headers.add('Forwarded', 'for=_3;by=_4')
+    req = make_mocked_request('GET', '/', headers=headers)
+    assert len(req.forwarded) == 4
+    assert req.forwarded[0]['for'] == '_1'
+    assert 'for' not in req.forwarded[1]
+    assert 'for' not in req.forwarded[2]
+    assert req.forwarded[3]['by'] == '_4'
+
+
+def test_multiple_forwarded_headers_injection():
+    headers = CIMultiDict()
+    # This could be sent by an attacker, hoping to "shadow" the second header.
+    headers.add('Forwarded', 'for=_injected;by="')
+    # This is added by our trusted reverse proxy.
+    headers.add('Forwarded', 'for=_real;by=_actual_proxy')
+    req = make_mocked_request('GET', '/', headers=headers)
+    assert len(req.forwarded) == 2
+    assert 'by' not in req.forwarded[0]
+    assert req.forwarded[1]['for'] == '_real'
+    assert req.forwarded[1]['by'] == '_actual_proxy'
+
+
+def test_host_by_host_header():
+    req = make_mocked_request('GET', '/',
+                              headers=CIMultiDict({'Host': 'example.com'}))
     assert req.host == 'example.com'
 
 
-def test_host_by_forwarded_header_malformed(make_request):
-    req = make_request('GET', '/',
-                       headers=CIMultiDict({'Forwarded': 'malformed value'}))
-    assert req.host is None
+def test_raw_headers():
+    req = make_mocked_request('GET', '/',
+                              headers=CIMultiDict({'X-HEADER': 'aaa'}))
+    assert req.raw_headers == ((b'X-HEADER', b'aaa'),)
 
 
-def test_host_by_x_forwarded_host_header(make_request):
-    req = make_request('GET', '/',
-                       headers=CIMultiDict(
-                           {'X-Forwarded-Host': 'example.com'}))
-    assert req.host == 'example.com'
-
-
-def test_host_by_host_header(make_request):
-    req = make_request('GET', '/',
-                       headers=CIMultiDict({'Host': 'example.com'}))
-    assert req.host == 'example.com'
-
-
-def test_raw_headers(make_request):
-    req = make_request('GET', '/',
-                       headers=CIMultiDict({'X-HEADER': 'aaa'}))
-    assert req.raw_headers == ((b'X-Header', b'aaa'),)
-
-
-def test_rel_url(make_request):
-    req = make_request('GET', '/path')
+def test_rel_url():
+    req = make_mocked_request('GET', '/path')
     assert URL('/path') == req.rel_url
 
 
-def test_url_url(make_request):
-    req = make_request('GET', '/path', headers={'HOST': 'example.com'})
+def test_url_url():
+    req = make_mocked_request('GET', '/path', headers={'HOST': 'example.com'})
     assert URL('http://example.com/path') == req.url
 
 
@@ -381,6 +400,13 @@ def test_clone():
     assert req2.rel_url == URL('/path')
 
 
+def test_clone_client_max_size():
+    req = make_mocked_request('GET', '/path', client_max_size=1024)
+    req2 = req.clone()
+    assert req._client_max_size == req2._client_max_size
+    assert req2._client_max_size == 1024
+
+
 def test_clone_method():
     req = make_mocked_request('GET', '/path')
     req2 = req.clone(method='POST')
@@ -487,3 +513,44 @@ def test_make_too_big_request_limit_None(loop):
                               client_max_size=max_size)
     txt = yield from req.read()
     assert len(txt) == 1024**2 + 1
+
+
+def test_remote_peername_tcp():
+    transp = mock.Mock()
+    transp.get_extra_info.return_value = ('10.10.10.10', 1234)
+    req = make_mocked_request('GET', '/', transport=transp)
+    assert req.remote == '10.10.10.10'
+
+
+def test_remote_peername_unix():
+    transp = mock.Mock()
+    transp.get_extra_info.return_value = '/path/to/sock'
+    req = make_mocked_request('GET', '/', transport=transp)
+    assert req.remote == '/path/to/sock'
+
+
+def test_save_state_on_clone():
+    req = make_mocked_request('GET', '/')
+    req['key'] = 'val'
+    req2 = req.clone()
+    req2['key'] = 'val2'
+    assert req['key'] == 'val'
+    assert req2['key'] == 'val2'
+
+
+def test_clone_scheme():
+    req = make_mocked_request('GET', '/')
+    req2 = req.clone(scheme='https')
+    assert req2.scheme == 'https'
+
+
+def test_clone_host():
+    req = make_mocked_request('GET', '/')
+    req2 = req.clone(host='example.com')
+    assert req2.host == 'example.com'
+
+
+def test_clone_remote():
+    req = make_mocked_request('GET', '/')
+    req2 = req.clone(remote='11.11.11.11')
+    assert req2.remote == '11.11.11.11'
